
-- BrakeSounds-Addon

-- Author: Thomas Heck (Patar)
-- Version: 1.0
-- Date: 22.05.16


BrakeSounds = {};

g_BSModDir = g_currentModDirectory;
	
function BrakeSounds.prerequisitesPresent(specializations)
    return true
end;

function BrakeSounds:loadMap(name)
	print("BrakeSounds V1.0 by Patar Loaded")
	

	-- load Sounds global
	closeBrakeTrailerSoundFile = Utils.getFilename("Sounds/closeBrakeTrailer.wav", g_BSModDir);
    self.closeBrakeTrailerSoundId = createSample("CloseBrakeTrailer");
    loadSample(self.closeBrakeTrailerSoundId, closeBrakeTrailerSoundFile, false);
	
	closeHandbrakeTrailerSoundFile = Utils.getFilename("Sounds/closeHandbrakeTrailer.wav", g_BSModDir);
    self.closeHandbrakeTrailerSoundId = createSample("CloseHandbrakeTrailer");
    loadSample(self.closeHandbrakeTrailerSoundId, closeHandbrakeTrailerSoundFile, false);
	
	releaseBrakeTrailerSoundFile = Utils.getFilename("Sounds/releaseBrakeTrailer.wav", g_BSModDir);
    self.releaseBrakeTrailerSoundId = createSample("ReleaseBrakeTrailer");
    loadSample(self.releaseBrakeTrailerSoundId, releaseBrakeTrailerSoundFile, false);
	
	closeBrakeSoundFile = Utils.getFilename("Sounds/closeBrake.wav", g_BSModDir);
    self.closeBrakeSoundId = createSample("closeBrake");
    loadSample(self.closeBrakeSoundId, closeBrakeSoundFile, false);
	
	releaseBrakeSoundFile = Utils.getFilename("Sounds/releaseBrake.wav", g_BSModDir);
    self.releaseBrakeSoundId = createSample("ReleaseBrake");
    loadSample(self.releaseBrakeSoundId, releaseBrakeSoundFile, false);
	
	releaseHandbrakeSoundFile = Utils.getFilename("Sounds/releaseHandbrake.wav", g_BSModDir);
    self.releaseHandbrakeSoundId = createSample("ReleaseHandbrake");
    loadSample(self.releaseHandbrakeSoundId, releaseHandbrakeSoundFile, false);
	
	-- load gui global
	self.hud = Overlay:new("BrakeSounds", Utils.getFilename("BrakeSounds_hud.dds", g_BSModDir), 0.323, 0.337, 0.36, 0.33);
	
	-- implement custom function to standard-specializations
	Attachable.update = Utils.appendedFunction(Attachable.update, BrakeSounds.updateT);
	Attachable.onAttach = Utils.appendedFunction(Attachable.onAttach, BrakeSounds.onAttachT);
	Steerable.update = Utils.appendedFunction(Steerable.update, BrakeSounds.updateTr);
	Steerable.onEnter = Utils.appendedFunction(Steerable.onEnter, BrakeSounds.onEnterTr);
	
	-- xml-file to save assigned vehicles and trailers
	BrakeSounds.global = {};
	BrakeSounds.global.assignedVehicles = {};
	BrakeSounds.global.oSA = {};
	
	
	
	
	
	-- load assigned vehicles
    local fileName = g_modsDirectory .. "/" .. "BrakeSounds_config.xml";
    if fileExists(fileName) then
		local tag = "BrakeSounds";
		local xmlFile = loadXMLFile(tag, fileName, tag);
		local i = 0;
		while true do
			tag = ("BrakeSounds.vehicle(%d)"):format(i);
			if not hasXMLProperty(xmlFile, tag.."#modname") then
				break;
			end
			i=i+1;
			local entry = {};
			entry.hasAirBrake = getXMLBool(xmlFile, tag.."#hasAirBrake");
			entry.modname = getXMLString(xmlFile, tag.."#modname");
			table.insert(BrakeSounds.global.assignedVehicles, entry);
        end;
		delete(xmlFile)
    end;

end;


function BrakeSounds:deleteMap()
	delete(self.closeBrakeTrailerSoundId);
	delete(self.closeHandbrakeTrailerSoundId);
	delete(self.releaseBrakeTrailerSoundId);
	delete(self.closeBrakeSoundId);
	delete(self.releaseBrakeSoundId);
	delete(self.releaseHandbrakeSoundId);
end;

function BrakeSounds:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BrakeSounds:keyEvent(unicode, sym, modifier, isDown)
end;

function BrakeSounds:update(dt)


	if table.getn(self.global.oSA) > 0 then
		local entry = {};
		entry.modname = self.global.oSA[1].configFileName;
		if InputBinding.hasEvent(InputBinding.BS_YES) then
			entry.hasAirBrake = true;
			self.global.oSA[1].self.hasAirBrake = true;
			table.insert(BrakeSounds.global.assignedVehicles, entry);
			BrakeSounds.saveXMLFile();
			if self.global.oSA[1].storePic ~= 0 then
				delete(self.global.oSA[1].storePic);
			end;
			table.remove(self.global.oSA, 1);
		elseif InputBinding.hasEvent(InputBinding.BS_NO) then
			entry.hasAirBrake = false;
			self.global.oSA[1].self.hasAirBrake = false;
			table.insert(BrakeSounds.global.assignedVehicles, entry);
			BrakeSounds.saveXMLFile();
			if self.global.oSA[1].storePic ~= 0 then
				delete(self.global.oSA[1].storePic);
			end;
			table.remove(self.global.oSA, 1);
		end;
	end;

end;

function BrakeSounds:updateTick(dt)
end;

function BrakeSounds:updateT(dt)

	if self.hasAirBrake then
		if self.BS == nil then
			self.BS = {pressureState=false, handbrakeState=false}
		end;
		
		axisForward = -InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
		if InputBinding.isAxisZero(axisForward) then
			axisForward = -InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE)
		end;
		
		if self:getIsActiveForSound() then
		
			if self.attacherVehicle ~= nil then
				if self.attacherVehicle.driveControl ~= nil then
					if self.attacherVehicle:dCcheckModule("handBrake") then
						self.BS.handBrakeActive = self.attacherVehicle.driveControl.handBrake.isActive;
					else
						self.BS.handBrakeActive = false;
					end;
				else
					self.BS.handBrakeActive = false;
				end;
			else
				self.BS.handBrakeActive = false;
			end;
			-- trailer brake
			if self.BS.pressureState == true and axisForward >= 0 and not self.BS.handBrakeActive then
				playSample(BrakeSounds.releaseBrakeTrailerSoundId, 1, 1, 0);
				self.BS.pressureState = false;
			end;
			if not self.BS.pressureState and axisForward < -0.3 and not self.BS.handBrakeActive then
				playSample(BrakeSounds.closeBrakeTrailerSoundId, 1, 0.5, 0);
				self.BS.pressureState = true;
			end;
			
			-- trailer handbrake
			if self.BS.handbrakeState == true and not self.BS.handBrakeActive then
				playSample(BrakeSounds.releaseBrakeTrailerSoundId, 1, 1, 0);
				self.BS.handbrakeState = false;
			end;
			if not self.BS.handbrakeState and self.BS.handBrakeActive then
				playSample(BrakeSounds.closeHandbrakeTrailerSoundId, 1, 1, 0);
				self.BS.handbrakeState = true;
			end;
		end;
	end;
end;

function BrakeSounds:updateTr(dt)

	if self.hasAirBrake then
		if self.BS == nil then
			self.BS = {pressureState=false, handbrakeState=false}
		end;
		
		axisForward = -InputBinding.getDigitalInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE);
		if InputBinding.isAxisZero(axisForward) then
			axisForward = -InputBinding.getAnalogInputAxis(InputBinding.AXIS_MOVE_FORWARD_VEHICLE)
		end;
		
		if self:getIsActiveForSound() then
		
			if self.driveControl ~= nil then
				if self:dCcheckModule("handBrake") then
					self.BS.handBrakeActive = self.driveControl.handBrake.isActive;
				else
					self.BS.handBrakeActive = false;
				end;
			else
				self.BS.handBrakeActive = false;
			end;
			
			-- tractor brake
			if self.BS.pressureState == true and axisForward >= 0 and not self.BS.handBrakeActive then
				playSample(BrakeSounds.releaseBrakeSoundId, 1, 1, 0);
				self.BS.pressureState = false;
			end;
			if not self.BS.pressureState and axisForward < -0.3 and not self.BS.handBrakeActive then
				playSample(BrakeSounds.closeBrakeSoundId, 1, 1, 0);
				self.BS.pressureState = true;
			end;
			
			-- tractor handbrake
			if self.BS.handbrakeState == true and not self.BS.handBrakeActive then
				playSample(BrakeSounds.releaseHandbrakeSoundId, 1, 1, 0);
				self.BS.handbrakeState = false;
			end;
			if not self.BS.handbrakeState and self.BS.handBrakeActive then
				playSample(BrakeSounds.closeBrakeSoundId, 1, 1, 0);
				self.BS.handbrakeState = true;
			end;
		end;
	end;
end;

function BrakeSounds:draw()
	if table.getn(self.global.oSA) > 0 then
		self.hud:render();
		if self.global.oSA[1].storePic ~= 0 then
			renderOverlay(self.global.oSA[1].storePic ,0.356,0.392,0.095,0.152);
		end;
		local cplstr = g_i18n:getText("BS_AIRBREAK_avail");
		local nbr = string.find(cplstr, " ");
		local scndlne = string.sub(cplstr, nbr+1)
		local frstlne = string.gsub(cplstr, scndlne, "");
		setTextColor(0,0,0,1); renderText(0.487, 0.508, 0.03, string.format("%s\n%s%s", tostring(frstlne), tostring(scndlne), "?")); setTextColor(1,1,1,1); 
	end;
end;

function BrakeSounds:onEnterTr()
	local configFileName = self.configFileName;
		while true do
			local stf1, stf2 = string.find(tostring(configFileName), "/");
			if stf1 == nil then
				break;
			end;
			configFileName = string.sub(tostring(configFileName), stf1+1);
		end;
	local found = false;
	for k,v in pairs(BrakeSounds.global.assignedVehicles) do
		if v.modname == configFileName then
			found = true;
			self.hasAirBrake = v.hasAirBrake;
		end;
	end;
	if not found then
		local entry = {};
		entry.configFileName = configFileName;
		local storeItem = StoreItemsUtil.storeItemsByXMLFilename[self.configFileName:lower()];
		entry.storePic = createImageOverlay(storeItem.imageActive)
		entry.self = self;
		table.insert(BrakeSounds.global.oSA, entry);
	end;
end;

function BrakeSounds:onAttachT()

	if table.getn(self.wheels) > 1 then
		local configFileName = self.configFileName;
			while true do
				local stf1, stf2 = string.find(tostring(configFileName), "/");
				if stf1 == nil then
					break;
				end;
				configFileName = string.sub(tostring(configFileName), stf1+1);
			end;
		local found = false;
		for k,v in pairs(BrakeSounds.global.assignedVehicles) do
			if v.modname == configFileName then
				found = true;
				self.hasAirBrake = v.hasAirBrake;
			end;
		end;
		if not found then
			local entry = {};
			entry.configFileName = configFileName;
			local storeItem = StoreItemsUtil.storeItemsByXMLFilename[self.configFileName:lower()];
			entry.storePic = createImageOverlay(storeItem.imageActive)
			entry.self = self;
			table.insert(BrakeSounds.global.oSA, entry);
		end;
	end;
end;

function BrakeSounds.saveXMLFile()
    local fileName = g_modsDirectory .. "/" .. "BrakeSounds_config.xml";
    local tag = "BrakeSounds"
    local xmlFile = createXMLFile(tag, fileName, tag)
    local i = 0
    for k,v in pairs(BrakeSounds.global.assignedVehicles) do
      local tag = ("BrakeSounds.vehicle(%d)"):format(i)
      i=i+1
      setXMLString(xmlFile, tag.."#modname", v.modname)
      setXMLBool(xmlFile, tag.."#hasAirBrake", v.hasAirBrake)
    end;
    saveXMLFile(xmlFile);
    delete(xmlFile);
end;



addModEventListener(BrakeSounds);


















